@@ -10,8 +10,11 @@ from .interfaces.dataframe import IDataFrame, IDataFrameWriter, IDataFrameReader
 from .interfaces.session import ISession, ISparkContext, ICatalog
 from .interfaces.storage import IStorageManager, ITable, ISchema
 from .interfaces.functions import IFunction, IColumnFunction, IAggregateFunction
+from .schema_inference import SchemaInferenceEngine, infer_schema_from_data
+from .data_validation import DataValidator, validate_data, coerce_data
 
 __all__ = [
+    # Interfaces
     "IDataFrame",
     "IDataFrameWriter",
     "IDataFrameReader",
@@ -24,4 +27,11 @@ __all__ = [
     "IFunction",
     "IColumnFunction",
     "IAggregateFunction",
+    # Schema Inference
+    "SchemaInferenceEngine",
+    "infer_schema_from_data",
+    # Data Validation
+    "DataValidator",
+    "validate_data",
+    "coerce_data",
 ]
