@@ -60,6 +60,8 @@ class MockSparkSession:
         validation_mode: str = "relaxed",
         enable_type_coercion: bool = True,
         enable_lazy_evaluation: bool = True,
+        max_memory: str = "1GB",
+        allow_disk_spillover: bool = False,
     ):
         """Initialize MockSparkSession.
 
@@ -67,9 +69,17 @@ class MockSparkSession:
             app_name: Application name for the Spark session.
             validation_mode: "strict", "relaxed", or "minimal" validation behavior.
             enable_type_coercion: Whether to coerce basic types during DataFrame creation.
+            enable_lazy_evaluation: Whether to enable lazy evaluation (default True).
+            max_memory: Maximum memory for DuckDB to use (e.g., '1GB', '4GB', '8GB').
+                       Default is '1GB' for test isolation.
+            allow_disk_spillover: If True, allows DuckDB to spill to disk when memory is full.
+                                 If False (default), disables spillover for test isolation.
         """
         self.app_name = app_name
-        self.storage = DuckDBStorageManager()
+        self.storage = DuckDBStorageManager(
+            max_memory=max_memory,
+            allow_disk_spillover=allow_disk_spillover,
+        )
         from typing import cast
         from ...core.interfaces.storage import IStorageManager
 
@@ -101,10 +111,10 @@ class MockSparkSession:
             enable_lazy_evaluation=enable_lazy_evaluation,
         )
 
-        # Memory tracking (Phase 3)
-        self._tracked_dataframes: List[MockDataFrame] = []
-        self._approx_memory_usage_bytes: int = 0
-        self._benchmark_results: Dict[str, Dict[str, Any]] = {}
+        # Performance and memory tracking (delegated to SessionPerformanceTracker)
+        from ..performance_tracker import SessionPerformanceTracker
+
+        self._performance_tracker = SessionPerformanceTracker()
 
     @property
     def appName(self) -> str:
@@ -206,56 +216,46 @@ class MockSparkSession:
                 data = reordered_data
 
         if schema is None:
-            # Infer schema from data
+            # Infer schema from data using SchemaInferenceEngine
             if not data:
                 # For empty dataset, create empty schema
                 schema = MockStructType([])
             else:
-                # Simple schema inference
+                # Check if data is in expected format
                 sample_row = data[0]
                 if not isinstance(sample_row, (dict, tuple)):
                     raise IllegalArgumentException("Data must be a list of dictionaries or tuples")
 
-                fields = []
                 if isinstance(sample_row, dict):
-                    # Dictionary format - sort keys alphabetically to match PySpark behavior
-                    sorted_keys = sorted(sample_row.keys())
-                    for key in sorted_keys:
-                        value = sample_row[key]
-                        field_type = self._infer_type(value)
-                        fields.append(MockStructField(key, field_type))
+                    # Use SchemaInferenceEngine for dictionary data
+                    from ...core.schema_inference import SchemaInferenceEngine
 
-                schema = MockStructType(fields)
-
-                # Reorder data rows to match schema (alphabetical key order)
-                if isinstance(sample_row, dict):
-                    reordered_data = []
-                    for row in data:
-                        if isinstance(row, dict):
-                            reordered_row = {key: row[key] for key in sorted_keys}
-                            reordered_data.append(reordered_row)
-                        else:
-                            reordered_data.append(row)
-                    data = reordered_data
+                    schema, data = SchemaInferenceEngine.infer_from_data(data)
                 elif isinstance(sample_row, tuple):
-                    # Convert tuples to dictionaries using schema field names
-                    reordered_data = []
-                    field_names = [field.name for field in schema.fields]
-                    for row in data:
-                        if isinstance(row, tuple):
-                            row_dict = {field_names[i]: row[i] for i in range(len(row))}
-                            reordered_data.append(row_dict)
-                        else:
-                            reordered_data.append(row)
-                    data = reordered_data
+                    # For tuples, we need column names - this should have been handled earlier
+                    # If we get here, it's an error
+                    raise IllegalArgumentException(
+                        "Cannot infer schema from tuples without column names. "
+                        "Please provide schema or use list of column names."
+                    )
 
         # Apply validation and optional type coercion per mode
         if isinstance(schema, MockStructType) and data:
+            from ...core.data_validation import DataValidator
+
+            validator = DataValidator(
+                schema,
+                validation_mode=self._engine_config.validation_mode,
+                enable_coercion=self._engine_config.enable_type_coercion,
+            )
+
+            # Validate if in strict mode
             if self._engine_config.validation_mode == "strict":
-                self._validate_data_matches_schema(data, schema)
-            # In relaxed/minimal modes, perform optional light coercion
+                validator.validate(data)
+
+            # Coerce if enabled
             if self._engine_config.enable_type_coercion:
-                data = self._coerce_data_to_schema(data, schema)
+                data = validator.coerce(data)
 
         df = MockDataFrame(data, schema, self.storage)  # type: ignore[return-value]
         # Track memory usage for newly created DataFrame
@@ -267,181 +267,48 @@ class MockSparkSession:
 
     def _track_dataframe(self, df: MockDataFrame) -> None:
         """Track DataFrame for approximate memory accounting."""
-        self._tracked_dataframes.append(df)
-        self._approx_memory_usage_bytes += self._estimate_dataframe_size(df)
-
-    def _estimate_dataframe_size(self, df: MockDataFrame) -> int:
-        """Very rough size estimate based on rows, columns, and value sizes."""
-        num_rows = len(df.data)
-        num_cols = len(df.schema.fields)
-        # assume ~32 bytes per cell average (key+value overhead), adjustable
-        return num_rows * num_cols * 32
+        self._performance_tracker.track_dataframe(df)
 
     def get_memory_usage(self) -> int:
         """Return approximate memory usage in bytes for tracked DataFrames."""
-        return self._approx_memory_usage_bytes
+        return self._performance_tracker.get_memory_usage()
 
     def clear_cache(self) -> None:
         """Clear tracked DataFrames to free memory accounting."""
-        self._tracked_dataframes.clear()
-        self._approx_memory_usage_bytes = 0
+        self._performance_tracker.clear_cache()
 
-    # ---------------------------
-    # Benchmarking API (Phase 3)
-    # ---------------------------
     def benchmark_operation(self, operation_name: str, func: Any, *args: Any, **kwargs: Any) -> Any:
         """Benchmark an operation and record simple telemetry.
 
         Returns the function result. Records duration (s), memory_used (bytes),
         and result_size when possible.
         """
-        import time
-
-        start_mem = self.get_memory_usage()
-        start_time = time.time()
-        result = func(*args, **kwargs)
-        end_time = time.time()
-        end_mem = self.get_memory_usage()
-
-        size: int = 1
-        try:
-            if hasattr(result, "count"):
-                size = int(result.count())
-            elif hasattr(result, "collect"):
-                size = len(result.collect())
-            elif hasattr(result, "__len__"):
-                size = len(result)
-        except Exception:
-            size = 1
-
-        self._benchmark_results[operation_name] = {
-            "duration_s": max(end_time - start_time, 0.0),
-            "memory_used_bytes": max(end_mem - start_mem, 0),
-            "result_size": size,
-        }
-        return result
+        return self._performance_tracker.benchmark_operation(operation_name, func, *args, **kwargs)
 
     def get_benchmark_results(self) -> Dict[str, Dict[str, Any]]:
         """Return a copy of the latest benchmark results."""
-        return dict(self._benchmark_results)
+        return self._performance_tracker.get_benchmark_results()
 
     def _infer_type(self, value: Any) -> Any:
         """Infer data type from value.
 
+        Delegates to SchemaInferenceEngine for consistency.
+
         Args:
             value: Value to infer type from.
 
         Returns:
             Inferred data type.
         """
-        from ...spark_types import (
-            StringType,
-            LongType,
-            DoubleType,
-            BooleanType,
-            ArrayType,
-            MapType,
-        )
+        from ...core.schema_inference import SchemaInferenceEngine
 
-        if isinstance(value, bool):
-            return BooleanType()
-        elif isinstance(value, int):
-            return LongType()
-        elif isinstance(value, float):
-            return DoubleType()
-        elif isinstance(value, list):
-            # ArrayType - infer element type from first non-null element
-            element_type = StringType()  # Default to StringType
-            for item in value:
-                if item is not None:
-                    element_type = self._infer_type(item)
-                    break
-            return ArrayType(element_type)
-        elif isinstance(value, dict):
-            # MapType - assume string keys and string values for simplicity
-            return MapType(StringType(), StringType())
-        else:
-            return StringType()
+        return SchemaInferenceEngine._infer_type(value)
 
     # ---------------------------
     # Validation and Coercion
     # ---------------------------
-    def _validate_data_matches_schema(
-        self, data: List[Dict[str, Any]], schema: MockStructType
-    ) -> None:
-        """Validate that data rows conform to the provided schema.
-
-        Raises IllegalArgumentException on mismatches in strict mode.
-        """
-        field_types = {f.name: f.dataType.__class__.__name__ for f in schema.fields}
-        for row in data:
-            if not isinstance(row, dict):
-                raise IllegalArgumentException("Strict mode requires dict rows after normalization")
-            # Ensure all schema fields present
-            for name in field_types.keys():
-                if name not in row:
-                    raise IllegalArgumentException(f"Missing required field '{name}' in row")
-            # Type check best-effort
-            for name, value in row.items():
-                if name not in field_types:
-                    raise IllegalArgumentException(f"Unexpected field '{name}' in row")
-                expected = field_types[name]
-                if value is None:
-                    continue
-                actual_py = type(value).__name__
-                # Accept numeric widenings (int->LongType, float->DoubleType)
-                if expected in ("LongType", "IntegerType") and isinstance(value, int):
-                    continue
-                if expected in ("DoubleType", "FloatType") and isinstance(value, (int, float)):
-                    continue
-                if expected == "StringType" and isinstance(value, str):
-                    continue
-                if expected == "BooleanType" and isinstance(value, bool):
-                    continue
-                # For complex types, skip deep validation for now
-                # Otherwise raise
-                raise IllegalArgumentException(
-                    f"Type mismatch for field '{name}': expected {expected}, got {actual_py}"
-                )
-
-    def _coerce_data_to_schema(
-        self, data: List[Dict[str, Any]], schema: MockStructType
-    ) -> List[Dict[str, Any]]:
-        """Coerce data types to match schema when possible (best-effort)."""
-        coerced: List[Dict[str, Any]] = []
-        field_types = {f.name: f.dataType.__class__.__name__ for f in schema.fields}
-        for row in data:
-            if not isinstance(row, dict):
-                coerced.append(row)  # leave as-is if not normalized
-                continue
-            new_row: Dict[str, Any] = {}
-            for name in field_types.keys():
-                value = row.get(name)
-                expected = field_types[name]
-                new_row[name] = self._coerce_value(value, expected)
-            coerced.append(new_row)
-        return coerced
-
-    def _coerce_value(self, value: Any, expected_type_name: str) -> Any:
-        if value is None:
-            return None
-        try:
-            if expected_type_name in ("LongType", "IntegerType"):
-                return int(value)
-            if expected_type_name in ("DoubleType", "FloatType"):
-                return float(value)
-            if expected_type_name == "StringType":
-                return str(value)
-            if expected_type_name == "BooleanType":
-                if isinstance(value, bool):
-                    return value
-                if str(value).lower() in ("true", "1"):  # simple coercion
-                    return True
-                if str(value).lower() in ("false", "0"):
-                    return False
-        except Exception:
-            return value
-        return value
+    # NOTE: Validation and coercion logic extracted to DataValidator
+    # See: mock_spark/core/data_validation.py
 
     def sql(self, query: str) -> IDataFrame:
         """Execute SQL query (mockable version)."""
@@ -529,9 +396,19 @@ class MockSparkSession:
         return self.createDataFrame(data, ["id"])
 
     def stop(self) -> None:
-        """Stop the session."""
-        # Mock implementation - in real Spark this would stop the session
-        pass
+        """Stop the session and clean up resources."""
+        # Close DuckDB connections to prevent leaks between tests
+        try:
+            if hasattr(self, "storage") and hasattr(self.storage, "close"):
+                self.storage.close()
+        except Exception:
+            pass  # Ignore errors during cleanup
+
+        # Clear any cached data
+        try:
+            self.clear_cache()
+        except Exception:
+            pass
 
     def __enter__(self):
         """Context manager entry."""
